@@ -18,13 +18,14 @@ a Perl document stored in a specific named file.
 =cut
 
 use strict;
-use Carp         ();
-use Params::Util '_STRING';
-use base 'PPI::Document';
+use Carp          ();
+use Params::Util  qw{_STRING _INSTANCE};
+use PPI::Document ();
 
-use vars qw{$VERSION};
+use vars qw{$VERSION @ISA};
 BEGIN {
-	$VERSION = '1.203';
+	$VERSION = '1.220';
+	@ISA     = 'PPI::Document';
 }
 
 
@@ -50,8 +51,12 @@ Returns a new PPI::Document::File object, or C<undef> on error.
 
 sub new {
 	my $class    = shift;
-	my $filename = _STRING(shift)
-		or return $class->_error("Did not provide a file name to load");
+	my $filename = _STRING(shift);
+	unless ( defined $filename ) {
+		# Perl::Critic got a complaint about not handling a file
+		# named "0".
+		return $class->_error("Did not provide a file name to load");
+	}
 
 	# Load the Document
 	my $self = $class->SUPER::new( $filename, @_ ) or return undef;
@@ -59,7 +64,7 @@ sub new {
 	# Unlike a normal inheritance situation, due to our need to stay
 	# compatible with caching magic, this actually returns a regular
 	# anonymous document. We need to rebless if
-	if ( ref($self) eq 'PPI::Document' ) {
+	if ( _INSTANCE($self, 'PPI::Document') ) {
 		bless $self, 'PPI::Document::File';
 	} else {
 		die "PPI::Document::File SUPER call returned an object of the wrong type";
@@ -136,7 +141,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2001 - 2008 Adam Kennedy.
+Copyright 2001 - 2011 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
