@@ -33,16 +33,15 @@ that represent boolean contexts used in various conditions.
 C<PPI::Structure::Condition> has no methods beyond those provided by
 the standard L<PPI::Structure>, L<PPI::Node> and L<PPI::Element> methods.
 
-Got any ideas for methods? Submit a report to rt.cpan.org!
-
 =cut
 
 use strict;
-use base 'PPI::Structure';
+use PPI::Structure ();
 
-use vars qw{$VERSION};
+use vars qw{$VERSION @ISA};
 BEGIN {
-	$VERSION = '1.201';
+	$VERSION = '1.220';
+	@ISA     = 'PPI::Structure';
 }
 
 1;
@@ -59,7 +58,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2001 - 2006 Adam Kennedy.
+Copyright 2001 - 2011 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
